@@ -23,73 +23,6 @@ from google.auth.transport import requests as google_requests
 DEFAULT_PROXY_PORT = 3000
 
 
-class OAuthCallbackHandler(JupyterHandler):
-    """
-    Handler for /static/oauth-callback.html endpoint.
-    Serves the OAuth callback HTML file for Google authentication.
-    """
-    
-    async def get(self):
-        """Handle GET requests to serve the OAuth callback HTML"""
-        try:
-            # Get the path to the static directory
-            import os
-            import pkg_resources
-            
-            # Try to get the file from the installed package first
-            try:
-                # This works when the package is installed via pip
-                # The static files are installed in the labextension directory
-                callback_content = pkg_resources.resource_string('escobar', 'labextension/static/oauth-callback.html').decode('utf-8')
-                
-                self.log.info(f"🔐 CALLBACK: Serving OAuth callback from installed package")
-                
-                # Set proper headers
-                self.set_header('Content-Type', 'text/html; charset=UTF-8')
-                self.set_header('Cache-Control', 'no-cache, no-store, must-revalidate')
-                self.set_header('Pragma', 'no-cache')
-                self.set_header('Expires', '0')
-                
-                # Write the HTML content
-                self.write(callback_content)
-                self.log.info(f"🔐 CALLBACK: Successfully served OAuth callback HTML from package")
-                return
-                
-            except (ImportError, FileNotFoundError, OSError):
-                # Fallback to file system path for development
-                current_dir = os.path.dirname(os.path.abspath(__file__))
-                project_root = os.path.dirname(current_dir)
-                callback_file_path = os.path.join(project_root, 'static', 'oauth-callback.html')
-            
-            self.log.info(f"🔐 CALLBACK: Serving OAuth callback from: {callback_file_path}")
-            
-            # Check if file exists
-            if not os.path.exists(callback_file_path):
-                self.log.error(f"🔐 CALLBACK: OAuth callback file not found: {callback_file_path}")
-                self.set_status(404)
-                self.finish("OAuth callback file not found")
-                return
-            
-            # Read and serve the HTML file
-            with open(callback_file_path, 'r', encoding='utf-8') as f:
-                html_content = f.read()
-            
-            # Set proper headers
-            self.set_header('Content-Type', 'text/html; charset=UTF-8')
-            self.set_header('Cache-Control', 'no-cache, no-store, must-revalidate')
-            self.set_header('Pragma', 'no-cache')
-            self.set_header('Expires', '0')
-            
-            # Write the HTML content
-            self.write(html_content)
-            self.log.info(f"🔐 CALLBACK: Successfully served OAuth callback HTML")
-            
-        except Exception as e:
-            self.log.error(f"🔐 CALLBACK: Error serving OAuth callback: {e}")
-            self.set_status(500)
-            self.finish(f"Error serving OAuth callback: {str(e)}")
-
-
 class ProxyHandler(JupyterHandler):
     """
     Handler for /proxy endpoint.
@@ -220,6 +153,191 @@ class ProxyHandler(JupyterHandler):
         await self._proxy_request(path_with_port, 'OPTIONS')
 
 
+class OAuthCallbackHandler(JupyterHandler):
+    """
+    Handler for /static/escobar/oauth-callback.html endpoint.
+    Serves the OAuth callback HTML file for Google authentication.
+    """
+    
+    def _find_callback_file(self):
+        """Find the OAuth callback HTML file in various possible locations"""
+        import os
+        import sys
+        from pathlib import Path
+        
+        # List of possible locations to check
+        possible_paths = []
+        
+        # 1. Development mode: static directory in project root
+        current_dir = os.path.dirname(os.path.abspath(__file__))
+        project_root = os.path.dirname(current_dir)
+        dev_path = os.path.join(project_root, 'static', 'oauth-callback.html')
+        possible_paths.append(('Development', dev_path))
+        
+        # 2. Installed package: shared data directory
+        # The file should be in share/jupyter/labextensions/escobar/static/
+        try:
+            import jupyter_core.paths
+            data_dirs = jupyter_core.paths.jupyter_data_dir()
+            if isinstance(data_dirs, str):
+                data_dirs = [data_dirs]
+            elif not isinstance(data_dirs, list):
+                data_dirs = [data_dirs]
+            
+            for data_dir in data_dirs:
+                shared_path = os.path.join(data_dir, 'labextensions', 'escobar', 'static', 'oauth-callback.html')
+                possible_paths.append(('Jupyter shared data', shared_path))
+        except Exception as e:
+            self.log.debug(f"Could not get jupyter data dirs: {e}")
+        
+        # 3. Try to find via pkg_resources in the escobar package itself
+        try:
+            import pkg_resources
+            # Try different possible resource paths
+            resource_paths = [
+                'static/oauth-callback.html',
+                'labextension/static/oauth-callback.html'
+            ]
+            
+            for resource_path in resource_paths:
+                try:
+                    if pkg_resources.resource_exists('escobar', resource_path):
+                        # Return the content directly since pkg_resources handles the path
+                        content = pkg_resources.resource_string('escobar', resource_path).decode('utf-8')
+                        return ('Package resource', resource_path, content)
+                except Exception as e:
+                    self.log.debug(f"pkg_resources check failed for {resource_path}: {e}")
+                    
+        except ImportError:
+            self.log.debug("pkg_resources not available")
+        
+        # 4. Check if file is bundled with the Python package
+        try:
+            escobar_module_path = os.path.dirname(os.path.abspath(__file__))
+            bundled_path = os.path.join(escobar_module_path, 'static', 'oauth-callback.html')
+            possible_paths.append(('Bundled with package', bundled_path))
+        except Exception as e:
+            self.log.debug(f"Could not determine escobar module path: {e}")
+        
+        # Check each possible file path
+        for location_name, file_path in possible_paths:
+            self.log.info(f"🔐 CALLBACK: Checking {location_name}: {file_path}")
+            if os.path.exists(file_path):
+                try:
+                    with open(file_path, 'r', encoding='utf-8') as f:
+                        content = f.read()
+                    return (location_name, file_path, content)
+                except Exception as e:
+                    self.log.warning(f"🔐 CALLBACK: Could not read {file_path}: {e}")
+                    continue
+        
+        # If we get here, no file was found
+        self.log.error(f"🔐 CALLBACK: OAuth callback file not found in any of these locations:")
+        for location_name, file_path in possible_paths:
+            self.log.error(f"🔐 CALLBACK:   - {location_name}: {file_path}")
+        
+        return None
+    
+    async def get(self):
+        """Handle GET requests to serve the OAuth callback HTML"""
+        try:
+            # Find the callback file
+            result = self._find_callback_file()
+            
+            if result is None:
+                self.set_status(404)
+                self.finish("OAuth callback file not found")
+                return
+            
+            location_name, file_path, html_content = result
+            
+            self.log.info(f"🔐 CALLBACK: Serving OAuth callback from {location_name}: {file_path}")
+            
+            # Set proper headers
+            self.set_header('Content-Type', 'text/html; charset=UTF-8')
+            self.set_header('Cache-Control', 'no-cache, no-store, must-revalidate')
+            self.set_header('Pragma', 'no-cache')
+            self.set_header('Expires', '0')
+            
+            # Write the HTML content
+            self.write(html_content)
+            self.log.info(f"🔐 CALLBACK: Successfully served OAuth callback HTML")
+            
+        except Exception as e:
+            self.log.error(f"🔐 CALLBACK: Error serving OAuth callback: {e}")
+            self.set_status(500)
+            self.finish(f"Error serving OAuth callback: {str(e)}")
+
+
+class EnvironmentVariablesHandler(JupyterHandler):
+    """
+    Handler for /api/escobar/environment-variables endpoint.
+    Returns whitelisted environment variables as JSON.
+    """
+    
+    # Whitelist of allowed environment variables to expose
+    ALLOWED_ENV_VARS = [
+        'GCP_CLIENT_ID',
+        'GCP_PROJECT_ID',
+        'BONNIE_URL',
+        'ESCOBAR_THEME',
+        # Add more as needed
+    ]
+    
+    async def get(self):
+        """Handle GET requests to return environment variables as JSON"""
+        try:
+            print(f"[ESCOBAR-API] === ENVIRONMENT VARIABLES REQUEST ===")
+            print(f"[ESCOBAR-API] Client IP: {self.request.remote_ip}")
+            print(f"[ESCOBAR-API] Request path: {self.request.path}")
+            
+            # Collect whitelisted environment variables
+            variables = {}
+            found_count = 0
+            
+            for var_name in self.ALLOWED_ENV_VARS:
+                value = os.getenv(var_name, '').strip()
+                if value:
+                    variables[var_name] = value
+                    found_count += 1
+                    # Log with masked value for security
+                    masked_value = f"{value[:10]}...{value[-5:]}" if len(value) > 15 else value
+                    print(f"[ESCOBAR-API] ✅ {var_name}: {masked_value}")
+                else:
+                    print(f"[ESCOBAR-API] ❌ {var_name}: not found")
+            
+            # Prepare response
+            response_data = {
+                "success": True,
+                "variables": variables,
+                "timestamp": time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()),
+                "found_count": found_count,
+                "total_checked": len(self.ALLOWED_ENV_VARS)
+            }
+            
+            print(f"[ESCOBAR-API] Returning {found_count}/{len(self.ALLOWED_ENV_VARS)} environment variables")
+            
+            # Set proper headers
+            self.set_header('Content-Type', 'application/json')
+            self.set_header('Cache-Control', 'no-cache, no-store, must-revalidate')
+            self.set_header('Pragma', 'no-cache')
+            self.set_header('Expires', '0')
+            
+            # Write JSON response
+            self.write(json.dumps(response_data, indent=2))
+            print(f"[ESCOBAR-API] ✅ Successfully served environment variables")
+            
+        except Exception as e:
+            print(f"[ESCOBAR-API] ❌ Error serving environment variables: {e}")
+            self.set_status(500)
+            error_response = {
+                "success": False,
+                "error": str(e),
+                "timestamp": time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
+            }
+            self.write(json.dumps(error_response))
+
+
 class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
     """
     WebSocket proxy handler that forwards connections from /ws to target server
@@ -368,8 +486,6 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         print(f"[ESCOBAR-WS] Request headers: {dict(self.request.headers)}")
         print(f"[ESCOBAR-WS] Attempting to connect to target: {target_url}")
         
-        logging.info(f"WebSocket connection opened, proxying to {target_url}")
-        
         try:
             # Establish connection to target websocket server
             # Copy relevant headers from the original request
@@ -397,8 +513,8 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
                 target_url,
                 additional_headers=headers,
                 ssl=ssl_context,
-                ping_interval=20,
-                ping_timeout=10,
+                ping_interval=45,
+                ping_timeout=20,
                 max_size=100 * 1024 * 1024 
             )
             
@@ -410,7 +526,6 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
             asyncio.create_task(self._forward_from_target())
             
             print(f"[ESCOBAR-WS] === CONNECTION SETUP COMPLETE ===")
-            logging.info(f"Successfully connected to target websocket server: {target_url}")
             
         except Exception as e:
             end_time = time.time()
@@ -432,9 +547,16 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         """Called when a message is received from the client"""
         if self.target_ws and not self.is_closing:
             try:
+                # Check for ping messages and handle silently
+                try:
+                    data = json.loads(message)
+                    if data.get('method') == 'ping':
+                        return  # Silent ping handling - no logging, no forwarding
+                except (json.JSONDecodeError, TypeError):
+                    pass  # Not JSON or not a dict, continue with normal processing
+                
                 # Forward message to target server
                 await self.target_ws.send(message)
-                logging.debug(f"Forwarded message to target: {message[:100]}...")
             except Exception as e:
                 print(f"[ESCOBAR-WS] ERROR forwarding message to target:")
                 print(f"[ESCOBAR-WS]   Error: {str(e)}")
@@ -449,13 +571,10 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
                 if not self.is_closing:
                     # Forward message to client
                     self.write_message(message)
-                    logging.debug(f"Forwarded message from target: {message[:100]}...")
                 else:
                     break
                 
         except websockets.exceptions.ConnectionClosed as e:
-            logging.info(f"Target websocket connection closed - code: {getattr(e, 'code', 'NO_CODE')}, reason: {getattr(e, 'reason', 'NO_REASON')}")
-            
             if not self.is_closing:
                 self.close(code=1011, reason="Target server disconnected")
         except Exception as e:
@@ -476,8 +595,6 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         """Called when websocket connection is closed"""
         self.is_closing = True
         
-        logging.info("WebSocket connection closed")
-        
         # Close target connection if it exists
         if self.target_ws:
             asyncio.create_task(self._close_target_connection())
@@ -495,7 +612,6 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
                 
                 if not is_closed:
                     await self.target_ws.close()
-                    logging.info("Target websocket connection closed")
         except Exception as e:
             print(f"[ESCOBAR-WS] ERROR closing target connection:")
             print(f"[ESCOBAR-WS]   Error: {str(e)}")
@@ -503,6 +619,42 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
             logging.error(f"Error closing target websocket: {str(e)}")
 
 
+def _get_gcp_client_id_from_environment():
+    """
+    Get GCP_CLIENT_ID from system environment variable.
+    Returns tuple: (value, source_description)
+    """
+    print(f"[ESCOBAR-ENV] === SEARCHING FOR GCP_CLIENT_ID ===")
+    
+    # Check system environment variable
+    gcp_client_id = os.getenv('GCP_CLIENT_ID', '').strip()
+    
+    if gcp_client_id:
+        print(f"[ESCOBAR-ENV] ✅ Found GCP_CLIENT_ID in system environment")
+        print(f"[ESCOBAR-ENV] 🔑 Value: {gcp_client_id[:20]}...{gcp_client_id[-10:] if len(gcp_client_id) > 30 else gcp_client_id}")
+        return gcp_client_id, "system environment variable"
+    else:
+        print(f"[ESCOBAR-ENV] ❌ GCP_CLIENT_ID not found in system environment")
+    
+    # Show related environment variables for debugging
+    print(f"[ESCOBAR-ENV] === DEBUGGING: RELATED ENVIRONMENT VARIABLES ===")
+    gcp_related_vars = [(k, v) for k, v in os.environ.items() 
+                       if any(term in k.upper() for term in ['GCP', 'GOOGLE', 'CLIENT', 'OAUTH'])]
+    
+    if gcp_related_vars:
+        print(f"[ESCOBAR-ENV] Found {len(gcp_related_vars)} potentially related environment variables:")
+        for key, value in gcp_related_vars:
+            masked_value = f"{value[:10]}...{value[-5:]}" if len(value) > 15 else value
+            print(f"[ESCOBAR-ENV]   {key}={masked_value}")
+    else:
+        print(f"[ESCOBAR-ENV] No GCP/Google/Client/OAuth related environment variables found")
+    
+    print(f"[ESCOBAR-ENV] === GCP_CLIENT_ID SEARCH COMPLETE - NOT FOUND ===")
+    return None, None
+
+
+
+
 def setup_handlers(web_app):
     print(f"[ESCOBAR-WS] === SETTING UP HANDLERS ===")
     
@@ -512,51 +664,22 @@ def setup_handlers(web_app):
     print(f"[ESCOBAR-WS] Host pattern: {host_pattern}")
     print(f"[ESCOBAR-WS] Base URL: {base_url}")
     
-    # Debug web_app settings to find settings manager
-    print(f"[ESCOBAR-WS] Web app settings keys: {list(web_app.settings.keys())}")
-    
-    # Try to find the settings manager
-    settings_manager = None
-    
-    # Method 1: Check if it's directly in web_app.settings
-    if 'settings_manager' in web_app.settings:
-        settings_manager = web_app.settings['settings_manager']
-        print(f"[ESCOBAR-WS] Found settings manager in web_app.settings")
-    
-    # Method 2: Check if there's a serverapp with settings manager
-    elif hasattr(web_app, 'serverapp') and hasattr(web_app.serverapp, 'settings_manager'):
-        settings_manager = web_app.serverapp.settings_manager
-        print(f"[ESCOBAR-WS] Found settings manager in web_app.serverapp")
-    
-    # Method 3: Check if there's a parent app with settings manager
-    elif hasattr(web_app, 'parent') and hasattr(web_app.parent, 'settings_manager'):
-        settings_manager = web_app.parent.settings_manager
-        print(f"[ESCOBAR-WS] Found settings manager in web_app.parent")
-    
-    # Method 4: Try to import and get the settings manager directly
+    # Log environment variables for debugging (no settings modification)
+    gcp_client_id, source = _get_gcp_client_id_from_environment()
+    if gcp_client_id:
+        print(f"[ESCOBAR] ✅ GCP_CLIENT_ID available from {source}")
+        print(f"[ESCOBAR] 💡 Google Client ID will be available via REST endpoint")
     else:
-        try:
-            from jupyter_server.services.settings.manager import SettingsManager
-            # Try to create a settings manager instance
-            settings_manager = SettingsManager()
-            print(f"[ESCOBAR-WS] Created new settings manager instance")
-        except Exception as e:
-            print(f"[ESCOBAR-WS] Could not create settings manager: {e}")
-    
-    print(f"[ESCOBAR-WS] Settings manager: {settings_manager}")
-    print(f"[ESCOBAR-WS] Settings manager type: {type(settings_manager) if settings_manager else 'None'}")
-    
-    # Store the settings manager in web_app.settings for handlers to access
-    if settings_manager:
-        web_app.settings['settings_manager'] = settings_manager
-        print(f"[ESCOBAR-WS] Stored settings manager in web_app.settings")
-    else:
-        print(f"[ESCOBAR-WS] WARNING: No settings manager available - user bonnieUrl will not work")
+        print(f"[ESCOBAR] ❌ GCP_CLIENT_ID not found - will need manual configuration")
     
     # Register the /proxy endpoint with a path parameter
     proxy_pattern = url_path_join(base_url, "proxy", "(.*)")
     print(f"[ESCOBAR-WS] Proxy pattern: {proxy_pattern}")
     
+    # Register the environment variables API endpoint
+    env_vars_pattern = url_path_join(base_url, "api", "escobar", "environment-variables")
+    print(f"[ESCOBAR-WS] Environment variables pattern: {env_vars_pattern}")
+    
     # Register multiple WebSocket proxy endpoints that all route to the same backend
     ws_patterns = [
         url_path_join(base_url, "ws"),           # /ws
@@ -568,14 +691,21 @@ def setup_handlers(web_app):
     for pattern in ws_patterns:
         print(f"[ESCOBAR-WS]   - {pattern}")
     
-    # Register the OAuth callback endpoint - use escobar-specific path to avoid conflicts
-    oauth_callback_pattern = url_path_join(base_url, "escobar", "oauth-callback.html")
-    print(f"[ESCOBAR-WS] OAuth callback pattern: {oauth_callback_pattern}")
+    # Register OAuth callback endpoints with /static/escobar/ path
+    oauth_patterns = [
+        "/hub/static/escobar/oauth-callback.html",     # OAuth callback from Google (JupyterHub)
+        "/static/escobar/oauth-callback.html"          # Direct access (standalone or root)
+    ]
     
-    # Build handlers list with all WebSocket patterns
+    print(f"[ESCOBAR-WS] OAuth callback patterns:")
+    for pattern in oauth_patterns:
+        print(f"[ESCOBAR-WS]   - {pattern}")
+    
+    # Build handlers list with all endpoints
     handlers = [
         (proxy_pattern, ProxyHandler),
-        (oauth_callback_pattern, OAuthCallbackHandler),
+        (env_vars_pattern, EnvironmentVariablesHandler),
+        *[(pattern, OAuthCallbackHandler) for pattern in oauth_patterns],
         *[(pattern, WebSocketProxyHandler) for pattern in ws_patterns]
     ]
     
@@ -585,6 +715,8 @@ def setup_handlers(web_app):
     web_app.add_handlers(host_pattern, handlers)
     
     print(f"[ESCOBAR-WS] Handlers registered successfully")
+    print(f"[ESCOBAR-WS] Environment variables API endpoint:")
+    print(f"[ESCOBAR-WS]   - {env_vars_pattern}")
     print(f"[ESCOBAR-WS] WebSocket proxy endpoints available:")
     for pattern in ws_patterns:
         print(f"[ESCOBAR-WS]   - {pattern} → ws://localhost:8777/ws")
