@@ -6,7 +6,7 @@ import { Message } from '@lumino/messaging';
 import { ISettingRegistry } from '@jupyterlab/settingregistry';
 
 import { VoittaToolRouter } from "./voitta/voittaServer";
-import { initPythonBridge, callPython, registerFunction, get_ws, voittal_call_log, stopped_messages, PYTHON_CALL_EVENTS } from './voitta/pythonBridge_browser'
+import { initPythonBridge, callPython, registerFunction, get_ws, voittal_call_log, stopped_messages, PYTHON_CALL_EVENTS, stopKeepalivePinger } from './voitta/pythonBridge_browser'
 import {
   IContinueRequest,
   ICreateNewChatRequest,
@@ -392,6 +392,10 @@ export class ChatWidget extends Widget {
     // Initialize settings and start the application
     setTimeout(async () => {
       await this.initializeSettings();
+
+      // Auto-save JupyterHub username to settings if detected
+      await this.autoSaveJupyterHubUsername();
+
       await this.init();
     }, 100);
   }
@@ -509,6 +513,41 @@ export class ChatWidget extends Widget {
     (window as any).escobarCurrentSettings = settings;
   }
 
+  /**
+   * Auto-save JupyterHub username to settings if detected
+   */
+  private async autoSaveJupyterHubUsername(): Promise<void> {
+    try {
+      // Get the current detected username
+      const currentUsername = getUserName(this.settingsManager);
+
+      // Get the stored username from settings
+      const storedUsername = this.settingsManager.getLocalSettings().username;
+
+      // If JupyterHub username detected and different from stored settings
+      if (currentUsername !== 'User' && currentUsername !== storedUsername && currentUsername.includes('@')) {
+        console.log('🔧 AUTO-SAVE: Detected JupyterHub username:', currentUsername);
+        console.log('🔧 AUTO-SAVE: Current stored username:', storedUsername);
+
+        // Auto-save the detected JupyterHub username
+        await this.settingsManager.updateLocalSetting('username', currentUsername);
+
+        // Update global settings object
+        const updatedSettings = this.settingsManager.getCompleteSettings();
+        (window as any).escobarCurrentSettings = updatedSettings;
+        (window as any).escobarUsername = currentUsername;
+
+        console.log('🔧 AUTO-SAVE: Updated username in settings from JupyterHub:', currentUsername);
+      } else {
+        console.log('🔧 AUTO-SAVE: No JupyterHub username update needed');
+        console.log('🔧 AUTO-SAVE: Current username:', currentUsername);
+        console.log('🔧 AUTO-SAVE: Stored username:', storedUsername);
+      }
+    } catch (error) {
+      console.error('🔧 AUTO-SAVE: Failed to auto-save JupyterHub username:', error);
+    }
+  }
+
   /**
    * Handle settings changes from the SettingsManager
    */
@@ -553,6 +592,9 @@ export class ChatWidget extends Widget {
   }
 
   async init() {
+    // Stop any existing pinger before reinitializing
+    stopKeepalivePinger();
+
     await this.messageHandler.clearMessages();
     this.voittaToolRouter = new VoittaToolRouter();
     const tools = await get_tools(this.app, this.notebookTracker, this.debuggerService);
@@ -646,17 +688,34 @@ export class ChatWidget extends Widget {
       let selectedChatID = 'temp-session';
 
       if (this.availableChats.length > 0) {
-        // Use the last chat in the list
-        const lastChatIndex = this.availableChats.length - 1;
-        const lastChat = this.availableChats[lastChatIndex];
+        // Check if there's a saved current chat ID in settings
+        const currentSettings = this.settingsManager.getLocalSettings();
+        const savedChatId = currentSettings._currentChatId;
 
-        // Extract chatID from the chat object
-        // Note: Server returns 'chatId' (camelCase) but we use 'chatID' (capital D) internally
-        selectedChatID = lastChat.chatId;
+        if (savedChatId && savedChatId !== '') {
+          // Check if the saved chat ID exists in available chats
+          const savedChatExists = this.availableChats.some(chat => chat.chatId === savedChatId);
+
+          if (savedChatExists) {
+            selectedChatID = savedChatId;
+            console.log('🔄 CHAT: Using saved current chat ID:', selectedChatID);
+          } else {
+            console.log('🔄 CHAT: Saved chat ID not found in available chats, falling back to last chat');
+            // Fall back to last chat if saved chat doesn't exist
+            const lastChatIndex = this.availableChats.length - 1;
+            const lastChat = this.availableChats[lastChatIndex];
+            selectedChatID = lastChat.chatId;
+          }
+        } else {
+          // No saved chat ID, use the last chat in the list (original behavior)
+          const lastChatIndex = this.availableChats.length - 1;
+          const lastChat = this.availableChats[lastChatIndex];
+          selectedChatID = lastChat.chatId;
+        }
 
         if (!selectedChatID) {
           console.error('No chatID found in chat object! Chat object structure may be incorrect.');
-          console.log('Expected chatId property, but got:', lastChat);
+          console.log('Expected chatId property, but got:', this.availableChats[this.availableChats.length - 1]);
           // Fallback to temp-session if no chatID
           selectedChatID = 'temp-session';
         }
@@ -794,6 +853,14 @@ export class ChatWidget extends Widget {
     // Update the message handler's current chat ID
     this.messageHandler.setCurrentChatID(newChatID);
 
+    // Save the new chat ID to settings for persistence
+    try {
+      await this.settingsManager.updateLocalSetting('_currentChatId', newChatID);
+      console.log('🔄 CHAT: Saved new chat ID to settings:', newChatID);
+    } catch (error) {
+      console.error('🔄 CHAT: Failed to save new chat ID:', error);
+    }
+
     // Refresh the chat list to include the new chat
     await this.refreshChatList();
 
@@ -815,6 +882,14 @@ export class ChatWidget extends Widget {
     this.messageHandler.setCurrentChatID(chatId);
     console.log(`Selected chat: ${chatId}`);
 
+    // Save the current chat ID to settings for persistence
+    try {
+      await this.settingsManager.updateLocalSetting('_currentChatId', chatId);
+      console.log('🔄 CHAT: Saved current chat ID to settings:', chatId);
+    } catch (error) {
+      console.error('🔄 CHAT: Failed to save current chat ID:', error);
+    }
+
     // Update UI to reflect we're now in an existing chat
     this.updateChatStateInUI(chatId);
 
@@ -1009,6 +1084,9 @@ export class ChatWidget extends Widget {
     window.removeEventListener(PYTHON_CALL_EVENTS.START, this.boundDisableInput);
     window.removeEventListener(PYTHON_CALL_EVENTS.END, this.boundEnableInput);
 
+    // Stop keepalive pinger
+    stopKeepalivePinger();
+
     // Close WebSocket connection when widget is disposed
     const ws = get_ws();
     if (ws) {
