@@ -95,22 +95,6 @@ function extractSourceFromNotebook(notebookContent: any): string {
 
 // Enhanced validation and safety functions
 function validateFileContent(fileContent: any, filePath: string): string {
-    console.log(`=== DETAILED FILE ANALYSIS FOR ${filePath} ===`);
-    console.log(`Raw fileContent keys:`, Object.keys(fileContent || {}));
-    console.log(`Full fileContent object:`, JSON.stringify(fileContent, null, 2));
-
-    console.log(`Validating file content for ${filePath}:`, {
-        hasContent: !!fileContent,
-        contentType: typeof fileContent?.content,
-        contentLength: fileContent?.content?.length,
-        fileType: fileContent?.type,
-        format: fileContent?.format,
-        mimetype: fileContent?.mimetype,
-        writable: fileContent?.writable,
-        name: fileContent?.name,
-        path: fileContent?.path
-    });
-
     if (!fileContent) {
         throw new Error(`File content is null or undefined for ${filePath}`);
     }
@@ -125,36 +109,21 @@ function validateFileContent(fileContent: any, filePath: string): string {
     const fileType = fileContent.type || '';
     const format = fileContent.format || '';
 
-    console.log(`=== DECISION LOGIC FOR ${filePath} ===`);
-    console.log(`mimetype: "${mimetype}" (type: ${typeof mimetype})`);
-    console.log(`fileType: "${fileType}" (type: ${typeof fileType})`);
-    console.log(`format: "${format}" (type: ${typeof format})`);
-    console.log(`content type: ${typeof fileContent.content}`);
-    console.log(`mimetype.startsWith('text/'): ${mimetype.startsWith('text/')}`);
-    console.log(`mimetype === 'application/x-python-code': ${mimetype === 'application/x-python-code'}`);
-
     // PRIORITY 1: Handle text files based on mimetype (most reliable indicator)
     // This must come BEFORE checking fileType to handle Jupytext files correctly
     if (mimetype.startsWith('text/') || mimetype === 'application/x-python-code') {
-        console.log(`✅ TAKING TEXT FILE PATH for ${filePath} with mimetype: ${mimetype}`);
-
         if (typeof fileContent.content === 'string') {
-            console.log(`✅ Text file ${filePath} has string content, returning as-is`);
             return fileContent.content;
         } else if (typeof fileContent.content === 'object' && format === 'json') {
             // This is a Jupytext file - extract source from notebook cells
-            console.log(`✅ Extracting source from Jupytext file ${filePath} (type=${fileType}, format=${format})`);
-            console.log(`Notebook content structure:`, JSON.stringify(fileContent.content, null, 2));
             try {
                 const extractedSource = extractSourceFromNotebook(fileContent.content);
-                console.log(`✅ Successfully extracted source (${extractedSource.length} chars):`, extractedSource.substring(0, 200) + '...');
                 return extractedSource;
             } catch (err) {
-                console.error(`❌ Failed to extract source from Jupytext file ${filePath}:`, err);
+                console.error(`Failed to extract source from Jupytext file ${filePath}:`, err.message);
                 throw new Error(`Failed to extract source from Jupytext file ${filePath}: ${err.message}`);
             }
         } else {
-            console.error(`❌ Text file ${filePath} has unexpected content type: ${typeof fileContent.content}`);
             throw new Error(`Text file ${filePath} has unexpected content type: ${typeof fileContent.content}`);
         }
     }
@@ -169,15 +138,12 @@ function validateFileContent(fileContent: any, filePath: string): string {
         fileContent.content?.metadata?.jupytext) {
 
         console.log(`✅ DETECTED JUPYTEXT FILE with null mimetype: ${filePath}`);
-        console.log(`Jupytext metadata:`, JSON.stringify(fileContent.content.metadata.jupytext, null, 2));
-        console.log(`Notebook content structure:`, JSON.stringify(fileContent.content, null, 2));
 
         try {
             const extractedSource = extractSourceFromNotebook(fileContent.content);
-            console.log(`✅ Successfully extracted source from Jupytext (${extractedSource.length} chars):`, extractedSource.substring(0, 200) + '...');
             return extractedSource;
         } catch (err) {
-            console.error(`❌ Failed to extract source from Jupytext file with null mimetype ${filePath}:`, err);
+            console.error(`Failed to extract source from Jupytext file with null mimetype ${filePath}:`, err.message);
             throw new Error(`Failed to extract source from Jupytext file ${filePath}: ${err.message}`);
         }
     }
@@ -187,22 +153,18 @@ function validateFileContent(fileContent: any, filePath: string): string {
         !mimetype.startsWith('text/') &&
         !(filePath.endsWith('.py') && fileContent.content?.metadata?.jupytext)) {
 
-        console.log(`✅ TAKING NOTEBOOK FILE PATH for ${filePath} (actual notebook, not Jupytext)`);
         if (fileContent.content === null) {
             return '';
         }
         try {
             const jsonString = JSON.stringify(fileContent.content, null, 2);
-            console.log(`✅ Successfully stringified notebook content (${jsonString.length} chars)`);
             return jsonString;
         } catch (err) {
-            console.error(`❌ Failed to stringify notebook content for ${filePath}:`, err);
+            console.error(`Failed to stringify notebook content for ${filePath}:`, err.message);
             throw new Error(`Failed to stringify notebook content for ${filePath}: ${err.message}`);
         }
     }
 
-    console.log(`⚠️ No specific path matched, continuing to fallback logic...`);
-
     // Handle base64 encoded files
     if (format === 'base64' && typeof fileContent.content === 'string') {
         console.log(`Processing base64 file ${filePath}`);
@@ -280,42 +242,20 @@ function ensureString(value: any, context: string): string {
 async function readFileWithRetry(contents: any, filePath: string, maxRetries = 3): Promise<any> {
     for (let attempt = 1; attempt <= maxRetries; attempt++) {
         try {
-            const startTime = Date.now();
-            console.log(`Reading file ${filePath}, attempt ${attempt}/${maxRetries}`);
-
             const result = await contents.get(filePath);
-            const duration = Date.now() - startTime;
-
-            console.log(`File read attempt ${attempt} took ${duration}ms`, {
-                hasResult: !!result,
-                resultType: typeof result,
-                hasContent: !!result?.content,
-                contentType: typeof result?.content,
-                fileType: result?.type,
-                format: result?.format
-            });
 
             if (!result) {
                 throw new Error(`File read returned null/undefined on attempt ${attempt}`);
             }
 
-            // Don't validate content type here - let validateFileContent handle it
-            // This allows us to properly handle different content formats (string, object, etc.)
             return result;
         } catch (err) {
-            console.error(`File read attempt ${attempt} failed:`, {
-                error: err.message,
-                stack: err.stack,
-                filePath
-            });
-
             if (attempt === maxRetries) {
                 throw new Error(`Failed to read file ${filePath} after ${maxRetries} attempts: ${err.message}`);
             }
 
             // Progressive delay between retries
             const delay = 100 * attempt;
-            console.log(`Retrying in ${delay}ms...`);
             await new Promise(resolve => setTimeout(resolve, delay));
         }
     }
@@ -470,6 +410,10 @@ var fileOperationLocks = new Map<string, Promise<void>>();
 var lastOperationTime = new Map<string, number>();
 const MIN_OPERATION_INTERVAL = 150; // Minimum ms between operations on same file
 
+// Streaming content state management
+var streamingContent = new Map<string, string>();
+var streamingCallIds = new Map<string, string>(); // Maps filePath to current call_id
+
 // Network-resilient helper functions
 async function waitForFileOperationLock(filePath: string): Promise<void> {
     const existingLock = fileOperationLocks.get(filePath);
@@ -674,15 +618,15 @@ export function init_diff() {
                             const fileExtension = filePath.split('.').pop()?.toLowerCase() || '';
                             editorElement.classList.add(`language-${fileExtension}`);
 
-                            // Force a refresh of the editor view
-                            setTimeout(() => {
+                            // Force a refresh of the editor view using requestAnimationFrame for better performance
+                            requestAnimationFrame(() => {
                                 try {
                                     // Dispatch a dummy transaction to force a refresh
                                     editorView.dispatch({});
                                 } catch (err) {
                                     console.error(`Error refreshing editor ${index}:`, err);
                                 }
-                            }, 100);
+                            });
                         } else {
                             console.log(`Could not access view for editor ${index}`);
                         }
@@ -714,44 +658,80 @@ export function init_diff() {
                     replace: replace?.substring(0, 50) + (replace?.length > 50 ? '...' : '')
                 });
 
-                // Read the file content with network-resilient retry logic
-                let fileContent;
-                try {
-                    fileContent = await readFileWithNetworkResilience(contents, filePath);
-                } catch (err) {
-                    console.error(`File read failed for ${filePath}:`, {
-                        error: err.message,
-                        stack: err.stack
-                    });
-                    return JSON.stringify({
-                        "status": "fail",
-                        "message": `Cannot open file: ${filePath} - ${err.message}`,
-                        "stack": err.stack
-                    });
-                }
-
-                // Validate and extract content safely
+                // Handle streaming content state management
                 let originalContent: string;
-                try {
-                    originalContent = validateFileContent(fileContent, filePath);
-                } catch (err) {
-                    console.error(`File content validation failed for ${filePath}:`, {
-                        error: err.message,
-                        stack: err.stack,
-                        fileContent: typeof fileContent,
-                        hasContent: !!fileContent?.content
-                    });
-                    return JSON.stringify({
-                        "status": "fail",
-                        "message": `Invalid file content for ${filePath}: ${err.message}`,
-                        "stack": err.stack
-                    });
+                let isFirstOperation = false;
+
+                if (streamingContent.has(call_id)) {
+                    // Use cached content for subsequent operations
+                    originalContent = streamingContent.get(call_id);
+                    console.log(`📋 Using cached content for ${streaming ? 'streaming' : 'final'} operation (${originalContent.length} chars)`);
+                } else {
+                    // Read from disk only for the very first operation
+                    isFirstOperation = true;
+                    let fileContent;
+                    try {
+                        fileContent = await readFileWithNetworkResilience(contents, filePath);
+                    } catch (err) {
+                        console.error(`File read failed for ${filePath}:`, {
+                            error: err.message,
+                            stack: err.stack
+                        });
+                        return JSON.stringify({
+                            "status": "fail",
+                            "message": `Cannot open file: ${filePath} - ${err.message}`,
+                            "stack": err.stack
+                        });
+                    }
+
+                    // Validate and extract content safely
+                    try {
+                        originalContent = validateFileContent(fileContent, filePath);
+
+                        // Cache original content for the first operation
+                        streamingContent.set(call_id, originalContent);
+                        streamingCallIds.set(filePath, call_id);
+                        console.log(`💾 Cached original content for first operation (${originalContent.length} chars)`);
+                    } catch (err) {
+                        console.error(`File content validation failed for ${filePath}:`, {
+                            error: err.message,
+                            stack: err.stack,
+                            fileContent: typeof fileContent,
+                            hasContent: !!fileContent?.content
+                        });
+                        return JSON.stringify({
+                            "status": "fail",
+                            "message": `Invalid file content for ${filePath}: ${err.message}`,
+                            "stack": err.stack
+                        });
+                    }
                 }
 
                 // Apply the diff with safe string operations
                 let diffResult: string;
                 try {
-                    diffResult = safeStringReplace(originalContent, search, replace, filePath);
+                    // For streaming operations after the first one, treat replace as the complete new content
+                    if (streaming && !isFirstOperation && search === '+') {
+                        // For streaming append operations, the replace content is the complete result
+                        // Get the original file content (not the cached streaming content)
+                        const originalFileContent = streamingContent.get(call_id + '_original') || originalContent;
+                        if (!streamingContent.has(call_id + '_original')) {
+                            // Store the original file content separately for reference
+                            streamingContent.set(call_id + '_original', originalContent);
+                        }
+                        diffResult = originalFileContent + unescapeString(replace);
+                    } else if (!streaming && !isFirstOperation && search === '+') {
+                        // For final non-streaming operation, if we have streaming cache, use the complete result
+                        // The cached content already contains the complete final result from streaming
+                        diffResult = originalContent; // originalContent is already the final result from cache
+                        console.log(`🎯 Using final cached content directly (${diffResult.length} chars)`);
+                    } else {
+                        diffResult = safeStringReplace(originalContent, search, replace, filePath);
+                        // Store original content for streaming reference if this is the first operation
+                        if (isFirstOperation && search === '+') {
+                            streamingContent.set(call_id + '_original', originalContent);
+                        }
+                    }
                 } catch (err) {
                     console.error(`String replacement failed for ${filePath}:`, {
                         error: err.message,
@@ -764,6 +744,10 @@ export function init_diff() {
                     });
                 }
 
+                // Update cached content with the new result
+                streamingContent.set(call_id, diffResult);
+                console.log(`🔄 Updated cached content for ${streaming ? 'streaming' : 'final'} operation (${diffResult.length} chars)`);
+
                 const processingTime = Date.now() - startTime;
                 console.log(`File processing completed for ${filePath} in ${processingTime}ms`);
 
@@ -822,9 +806,10 @@ export function init_diff() {
 
                     container.appendChild(mergeView.dom);
 
-                    setTimeout(function () {
+                    // Apply syntax highlighting immediately using requestAnimationFrame for better performance
+                    requestAnimationFrame(() => {
                         applySyntaxHighlighting(mergeView);
-                    }, 500);
+                    });
 
                     // Create a widget to hold the container
                     const widget = new Widget();
@@ -842,34 +827,77 @@ export function init_diff() {
                 }
 
                 if (!streaming) {
-                    console.log(app.commands.listCommands());
-
-                    //app.commands.execute('docmanager:close', { path: filePath });
-
-
-                    if (mergeViews[call_id]) {
-                        // Find the widget by ID and close it
-                        const widget = mergeWidgets[call_id];
-                        //const widget = app.shell.widgets('main').find(w => w.id === call_id);
-                        if (widget) {
-                            setTimeout(() => {
-                                widget.close();
-                                setTimeout(() => {
-                                    app.commands.execute('docmanager:open', { path: filePath });
-                                    setTimeout(() => {
-                                        app.commands.execute('docmanager:reload')
-                                    }, 100)
-                                }, 100)
-                            }, 1000);
-                        }
-                    }
+                    // Clean up streaming cache when operation completes
+                    streamingContent.delete(call_id);
+                    streamingContent.delete(call_id + '_original'); // Clean up original content cache too
+                    streamingCallIds.delete(filePath);
+                    console.log(`🧹 Cleaned up streaming cache for ${filePath}`);
 
+                    // Save the file to disk (ensures persistence)
                     await contents.save(filePath, {
                         type: 'file',
                         format: 'text',
                         content: diffResult
                     });
 
+                    console.log(`💾 File saved successfully: ${filePath}`);
+
+                    // Update any open editors with the new content (without reloading from disk)
+                    try {
+                        const docManager = app.serviceManager.contents;
+                        const docRegistry = app.docRegistry;
+
+                        // Find any open widgets for this file
+                        const widgets = app.shell.widgets('main');
+                        for (const widget of widgets) {
+                            if (widget.title.label.includes(filePath.split('/').pop()) ||
+                                (widget as any).context?.path === filePath) {
+
+                                // Update the widget's content directly
+                                const context = (widget as any).context;
+                                if (context && context.model && typeof context.model.fromString === 'function') {
+                                    context.model.fromString(diffResult);
+
+                                    // Mark the document as saved to prevent "unsaved" state
+                                    if (context.model.dirty !== undefined) {
+                                        context.model.dirty = false;
+                                    }
+
+                                    // Trigger save state update if available
+                                    if (typeof context.model.stateChanged?.emit === 'function') {
+                                        context.model.stateChanged.emit({
+                                            name: 'dirty',
+                                            oldValue: true,
+                                            newValue: false
+                                        });
+                                    }
+
+                                    console.log(`📝 Updated open editor for ${filePath} and marked as saved`);
+                                }
+                            }
+                        }
+                    } catch (err) {
+                        console.warn(`Could not update open editors: ${err.message}`);
+                    }
+
+                    // Close the diff view and open/focus the file
+                    if (mergeViews[call_id]) {
+                        const widget = mergeWidgets[call_id];
+                        if (widget) {
+                            // Close the diff view
+                            widget.close();
+                            console.log(`🗑️ Closed diff view for ${filePath}`);
+
+                            // Open the file (or focus if already open)
+                            try {
+                                await app.commands.execute('docmanager:open', { path: filePath });
+                                console.log(`📂 Opened/focused file: ${filePath}`);
+                            } catch (err) {
+                                console.warn(`Could not open file: ${err.message}`);
+                            }
+                        }
+                    }
+
                     return JSON.stringify({
                         "status": "ok",
                         "new_content": diffResult
