@@ -584,12 +584,12 @@ export class MessageHandler {
           openai_api_key: this.openaiApiKey,
           anthropic_api_key: this.anthropicApiKey,
           gemini_api_key: this.geminiApiKey,
-          selected_provider: primaryModel, // Primary model
-          secondary_provider: secondaryProvider, // Secondary model
-          image_parse_provider: imageParseProvider // Image parsing model
+          selected_provider: primaryModel,
+          secondary_provider: secondaryProvider,
+          image_parse_provider: imageParseProvider
         };
 
-        const response = await callPython(userMessage, DEFAULT_TIMEOUT, true, true); // Show stop button for user messages
+        const response = await callPython(userMessage, DEFAULT_TIMEOUT, true, true);
         this.handlePythonResponse(response, responseMessage);
 
       } catch (error) {
