@@ -47,7 +47,7 @@ export class GoogleAuthManager {
     constructor(config: IGoogleAuthConfig) {
         this.clientId = config.clientId;
         this.scope = config.scope || 'openid email profile https://www.googleapis.com/auth/drive.readonly https://www.googleapis.com/auth/documents.readonly';
-        this.redirectUri = config.redirectUri || `${window.location.origin}/escobar/oauth-callback.html`;
+        this.redirectUri = config.redirectUri || `${window.location.origin}/static/escobar/oauth-callback.html`;
 
         // Validate client ID format
         if (!this.isValidClientId(this.clientId)) {
